"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const request = require("request");
const setCookie = require("set-cookie-parser");
const config_1 = require("./config");
const userData_1 = require("./userData");
const icloudSession_1 = require("./icloudSession");
const icloudRequestParams_1 = require("./icloudRequestParams");
class ICloudCalendar {
    constructor(userAgent, hostDomain, originAddress, refererAddress) {
        this.userAgent = userAgent ? userAgent : config_1.DEFAULT_USER_AGENT;
        this.hostDomain = hostDomain ? hostDomain : config_1.DEFAULT_HOST_DOMAIN;
        this.originAddress = originAddress ? originAddress : config_1.DEFAULT_ORIGIN_ADDRESS;
        this.refererAddress = refererAddress ? refererAddress : config_1.DEFAULT_REFERER_ADDRESS;
        this.request = request.defaults({
            headers: {
                'host': this.hostDomain,
                'origin': this.originAddress,
                'referer': this.refererAddress,
                'User-Agent': this.userAgent
            },
            jar: request.jar(),
            json: true
        });
        this.session = new icloudSession_1.ICloudSession();
    }
    login(appleId, password) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                if (!appleId || !password) {
                    reject('Invalid login credentials!');
                }
                else {
                    if (this.isConnected()) {
                        resolve(this.session.getParams());
                    }
                    else {
                        const requestConfig = {
                            url: `${config_1.SETUP_URL}${config_1.LOGIN_SUFFIX}`,
                            qs: this.session.getParams(),
                            json: new userData_1.UserData(appleId, password)
                        };
                        this.request.post(requestConfig, (err, resp, data) => {
                            if (err || data.error) {
                                reject('Invalid email/password combination.');
                            }
                            resolve(this.setConnectionData(data, resp));
                        });
                    }
                }
            });
        });
    }
    isConnected() {
        if (this.session.getExpire()) {
            var now = new Date();
            return now.getTime() < this.session.getExpire().getTime();
        }
        else {
            return false;
        }
    }
    getCalendars(language, timeZone) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const now = new Date();
                    const month = now.getMonth() + 1;
                    const today = `${now.getFullYear()}-${month < 10 ? `0${month}` : month}-${now.getDate()}`;
                    const calendarData = yield this.getCalendarData(language, timeZone, today, today);
                    if (calendarData && calendarData.Collection) {
                        resolve(calendarData.Collection);
                    }
                    else {
                        resolve({});
                    }
                }
                catch (err) {
                    reject(err);
                }
            }));
        });
    }
    getEvents(language, timeZone, startDate, endDate) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const calendarData = yield this.getCalendarData(language, timeZone, startDate, endDate);
                    if (calendarData && calendarData.Event) {
                        resolve(calendarData.Event);
                    }
                    else {
                        resolve({});
                    }
                }
                catch (err) {
                    reject(err);
                }
            }));
        });
    }
    getEventDetails(guid, pGuid, language, timeZone) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const validated = yield this.validateSession();
                    if (validated) {
                        if (!this.isConnected()) {
                            reject('Invalid connection to iCloud! Check X-APPLE-WEBAUTH-TOKEN existance');
                        }
                        else {
                            const params = this.generateRequestParams(language, timeZone, null, null);
                            const calendarData = yield this.fetchData(`${config_1.EVENT_DETAILS_WS_SUFFIX}/${pGuid}/${guid}`, params);
                            if (calendarData && calendarData.Event) {
                                resolve(calendarData.Event);
                            }
                            else {
                                resolve({});
                            }
                        }
                    }
                    else {
                        reject('Current session is not valid! Try to login first.');
                    }
                }
                catch (err) {
                    reject(err);
                }
            }));
        });
    }
    getCalendarData(language, timeZone, startDate, endDate) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                let validated;
                try {
                    validated = yield this.validateSession();
                }
                catch (err) {
                    reject(err);
                }
                if (validated) {
                    if (!this.isConnected()) {
                        reject('Invalid connection to iCloud! Check X-APPLE-WEBAUTH-TOKEN existance');
                    }
                    else {
                        const webServices = this.session.getWebServices();
                        if (!webServices || !webServices.calendar || webServices.calendar.status !== 'active' || !webServices.calendar.url) {
                            reject('iCloud Calendar Web Service Unavailable');
                        }
                        else {
                            try {
                                const params = this.generateRequestParams(language, timeZone, startDate, endDate);
                                const resp = yield this.fetchData(config_1.EVENTS_WS_SUFFIX, params);
                                resolve(resp);
                            }
                            catch (err) {
                                reject('Error fetching data! Try to login first.');
                            }
                        }
                    }
                }
                else {
                    reject('Current session is not valid! Try to login first.');
                }
            }));
        });
    }
    setConnectionData(data, response) {
        if (data && data.dsInfo) {
            this.session.setParamValue('dsid', data.dsInfo.dsid);
            if (data.webservices) {
                this.session.setWebServices(data.webservices);
            }
            const cookies = setCookie.parse(response, {
                decodeValues: true
            });
            for (let i = 0; i < cookies.length; i++) {
                if (cookies[i].expires) {
                    this.session.setExpire(new Date(cookies[i].expires));
                }
            }
            return true;
        }
        else {
            this.session.setExpire(null);
            this.session.setParamValue('dsid', undefined);
            this.session.setWebServices(undefined);
            return false;
        }
    }
    generateRequestParams(language, timeZone, startDate, endDate) {
        const dsid = this.session.getParams()['dsid'];
        return new icloudRequestParams_1.ICloudRequestParams(dsid, language, timeZone, startDate, endDate);
    }
    validateSession() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                const requestConfig = {
                    url: `${config_1.SETUP_URL}${config_1.VALIDATE_SUFFIX}`,
                    qs: this.session.getParams()
                };
                this.request.get(requestConfig, (err, resp, data) => {
                    if (err) {
                        this.setConnectionData(null, null);
                        reject(err);
                    }
                    else {
                        this.setConnectionData(data, resp);
                        resolve(true);
                    }
                });
            });
        });
    }
    fetchData(serviceResource, params) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                try {
                    const webServiceUrl = this.session.getWebServices().calendar.url.replace(':443', '');
                    const hostAddress = this.session.getWebServices().calendar.url.split('//')[1].split(':')[0];
                    const requestConfig = {
                        url: `${webServiceUrl}${serviceResource}`,
                        qs: params,
                        headers: {
                            host: hostAddress
                        }
                    };
                    this.request.get(requestConfig, (err, resp, body) => {
                        if (err) {
                            reject(err);
                        }
                        else {
                            resolve(body);
                        }
                    });
                }
                catch (err) {
                    reject(err);
                }
            });
        });
    }
}
exports.ICloudCalendar = ICloudCalendar;
//# sourceMappingURL=icloudCalendar.js.map